Package: puma
Type: Package
Title: Propagating Uncertainty in Microarray Analysis(including
        Affymetrix tranditional 3' arrays and exon arrays and Human
        Transcriptome Array 2.0)
Version: 3.54.0
Date: 2015-7-29
Author: Richard D. Pearson, Xuejun Liu, Magnus Rattray, Marta Milo, Neil D.
    Lawrence, Guido Sanguinetti, Li Zhang
Maintainer: Xuejun Liu <xuejun.liu@nuaa.edu.cn>
Depends: R (>= 3.2.0), oligo (>= 1.32.0),graphics,grDevices, methods,
        stats, utils, mclust, oligoClasses
Imports: Biobase (>= 2.5.5), affy (>= 1.46.0), affyio, oligoClasses
Suggests: pumadata, affydata, snow, limma, ROCR,annotate
Description: Most analyses of Affymetrix GeneChip data (including tranditional 3' arrays and exon arrays and Human Transcriptome Array 2.0) are based on point estimates of expression levels and ignore the uncertainty of such estimates. By propagating uncertainty to downstream analyses we can improve results from microarray analyses. For the first time, the puma package makes a suite of uncertainty propagation methods available to a general audience. In additon to calculte gene expression from Affymetrix 3' arrays, puma also provides methods to process exon arrays and produces gene and isoform expression for alternative splicing study. puma also offers improvements in terms of scope and speed of execution over previously available uncertainty propagation methods. Included are summarisation, differential expression detection, clustering and PCA methods, together with useful plotting functions.
License: LGPL
biocViews: Microarray, OneChannel, Preprocessing,
        DifferentialExpression, Clustering, ExonArray, GeneExpression,
        mRNAMicroarray, ChipOnChip, AlternativeSplicing,
        DifferentialSplicing, Bayesian, TwoChannel, DataImport, HTA2.0
URL: http://umber.sbs.man.ac.uk/resources/puma
Packaged: 2026-04-29 00:00:56 UTC; biocbuild
NeedsCompilation: yes
git_url: https://git.bioconductor.org/packages/puma
git_branch: RELEASE_3_23
git_last_commit: dbcf914
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
