# Security Policy

## Supported Versions

| Version       | Supported  |
| :------------ | :--------: |
| Latest release in 8.x | ✅ |
| Latest release in 7.x | ✅ |
| All other releases    | ❌ |

## Reporting a Vulnerability

[Report on Github](https://github.com/puma/puma/security) or contact [Evan Phoenix.](https://github.com/evanphx)
