## Updating Puma test cert files

One must have openssl and keytool in their Path.  Keytool is part of the JDK.

It maybe helpful to install the ['certificate_authority'](https://github.com/cchandler/certificate_authority) gem.

Then run 'examples/generate_all_certs' from a bash shell.
