\name{eset_mmgmos}
\alias{eset_mmgmos}
\docType{data}
\title{ An example ExpressionSet created from the Dilution data with mmgmos}
\description{
  This data is created by applying mmgmos to the Dilution AffyBatch object
  from the affydata package.
}
\usage{data(eset_mmgmos)}
\format{
  An object of class \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}}.
}
\source{
  see \code{\link[affydata:Dilution]{Dilution}}
}
\keyword{datasets}
