\name{pplrUnsorted}
\alias{pplrUnsorted}
\title{Return an unsorted matrix of PPLR values}
\description{
Returns the output from \code{\link{pplr}} as an unsorted matrix (i.e. sorted according to the original sorting in the original matrix)
}
\usage{
pplrUnsorted(p)
}
\arguments{
	\item{p}{ A matrix as output by \code{\link{pplr}}. }
}
\value{
  A matrix of PPLR values
}
\author{ Richard D. Pearson }
\seealso{Related method \code{\link{pplr}}}
\keyword{manip}
